@@ -138,17 +138,17 @@ class ErrorHandling(BrowserView):
         # ['', 'Plone', 'folder', 'item', '@@view', 'param']
         #                                ^
         splitpoint = len(old_path_elements)
-
         while splitpoint > 1:
             possible_obj_path = "/".join(old_path_elements[:splitpoint])
             remainder = old_path_elements[splitpoint:]
             new_path = storage.get(possible_obj_path)
 
             if new_path:
-                if new_path == possible_obj_path:
+                if new_path.startswith(possible_obj_path):
                     # New URL would match originally requested URL.
                     # Lets not cause a redirect loop.
                     return None
+
                 return new_path + "/" + "/".join(remainder)
 
             splitpoint -= 1
@@ -162,7 +162,7 @@ class ErrorHandling(BrowserView):
 
         This method is based on FourOhFourView.attempt_redirect() from
         p.a.redirector. It's copied here because we want to answer redirects
-        to non-GET methods with status 308, but since this method locks the
+        to non-GET methods with status 307, but since this method locks the
         response status, we wouldn't be able to change it afterwards.
         """
         url = self._url()
@@ -178,6 +178,12 @@ class ErrorHandling(BrowserView):
         if storage is None:
             return False
 
+        # remove ++api++ traverser
+        if "++api++" in old_path_elements:
+            api_traverser_pos = old_path_elements.index("++api++")
+            old_path_elements = [el for el in old_path_elements if el != "++api++"]
+        else:
+            api_traverser_pos = None
         old_path = "/".join(old_path_elements)
 
         # First lets try with query string in cases or content migration
@@ -211,28 +217,32 @@ class ErrorHandling(BrowserView):
             url_path = quote(url_path)
             url = urllib.parse.SplitResult(*(url[:2] + (url_path,) + url[3:])).geturl()
         else:
+            # reinsert ++api++ traverser
+            if api_traverser_pos is not None:
+                new_path_elements = new_path.split("/")
+                new_path_elements.insert(api_traverser_pos, "++api++")
+                new_path = "/".join(new_path_elements)
             url = self.request.physicalPathToURL(new_path)
 
         # some analytics programs might use this info to track
         if query_string:
             url += "?" + query_string
 
-        # Answer GET requests with 301. Every other method will be answered
-        # with 308 Permanent Redirect, which instructs the client to NOT
+        # Answer GET requests with 302. Every other method will be answered
+        # with 307 Temporary Redirect, which instructs the client to NOT
         # switch the method (if the original request was a POST, it should
         # re-POST to the new URL from the Location header).
         if self.request.method.upper() == "GET":
-            status = 301
+            status = 302
         else:
-            status = 308
+            status = 307
 
         self.request.response.redirect(url, status=status, lock=1)
         return True
 
     @memoize
     def _url(self):
-        """Get the current, canonical URL
-        """
+        """Get the current, canonical URL"""
         return self.request.get(
             "ACTUAL_URL",
             self.request.get(
